@@ -38,11 +38,7 @@ class Query_Info {
 
 	MYSQL_STMT *mysql_stmt;
 	stmt_execute_metadata_t *stmt_meta;
-#ifndef PROXYSQL_STMT_V14
-	uint32_t stmt_global_id;
-#else
 	uint64_t stmt_global_id;
-#endif
 	MySQL_STMT_Global_info *stmt_info;
 
 	int QueryLength;
@@ -147,6 +143,7 @@ class MySQL_Session
 
 	uint32_t thread_session_id;
 	unsigned long long last_insert_id;
+	int last_HG_affected_rows;
 	enum session_status status;
 	int healthy;
 	int user_max_connections;
