@@ -666,6 +666,7 @@ __thread int mysql_thread___monitor_ping_max_failures;
 __thread int mysql_thread___monitor_ping_timeout;
 __thread int mysql_thread___monitor_read_only_interval;
 __thread int mysql_thread___monitor_read_only_timeout;
+__thread int mysql_thread___monitor_read_only_max_timeout_count;
 __thread bool mysql_thread___monitor_wait_timeout;
 __thread bool mysql_thread___monitor_writer_is_also_reader;
 __thread int mysql_thread___monitor_replication_lag_interval;
@@ -771,6 +772,7 @@ extern __thread int mysql_thread___monitor_ping_max_failures;
 extern __thread int mysql_thread___monitor_ping_timeout;
 extern __thread int mysql_thread___monitor_read_only_interval;
 extern __thread int mysql_thread___monitor_read_only_timeout;
+extern __thread int mysql_thread___monitor_read_only_max_timeout_count;
 extern __thread bool mysql_thread___monitor_wait_timeout;
 extern __thread bool mysql_thread___monitor_writer_is_also_reader;
 extern __thread int mysql_thread___monitor_replication_lag_interval;
