@@ -1,40 +1,9 @@
-#include "btree_map.h"
+//#include "btree_map.h"
 #include "proxysql.h"
 #include "cpp.h"
 #include "proxysql_atomic.h"
 #include "SpookyV2.h"
 
-/*
-typedef struct _account_details_t {
-	char *username;
-	char *password;
-	bool use_ssl;
-	int default_hostgroup;
-	char *default_schema;
-	bool schema_locked;
-	bool transaction_persistent;
-	bool fast_forward;
-} account_details_t;
-*/
-
-//#ifdef DEBUG
-//#define DEB "_DEBUG"
-//#else
-//#define DEB ""
-//#endif /* DEBUG */
-//#define MYSQL_AUTHENTICATION_VERSION "0.1.0706" DEB
-
-/*
-typedef btree::btree_map<uint64_t, account_details_t *> BtMap;
-
-typedef struct _creds_group_t {
-	rwlock_t lock;
-	BtMap bt_map;
-	PtrArray cred_array;
-} creds_group_t;
-*/
-
-
 MySQL_Authentication::MySQL_Authentication() {
 #ifdef DEBUG
 	if (glovars.has_debug==false) {
@@ -44,73 +13,53 @@ MySQL_Authentication::MySQL_Authentication() {
 		perror("Incompatible debagging version");
 		exit(EXIT_FAILURE);
 	}
+#ifdef PROXYSQL_AUTH_PTHREAD_MUTEX
+	pthread_rwlock_init(&creds_backends.lock, NULL);
+	pthread_rwlock_init(&creds_frontends.lock, NULL);
+#else
 	spinlock_rwlock_init(&creds_backends.lock);
 	spinlock_rwlock_init(&creds_frontends.lock);
+#endif
 	creds_backends.cred_array = new PtrArray();
 	creds_frontends.cred_array = new PtrArray();
-
-//	spinlock_rwlock_init(&rwlock);
-//	authdb=new SQLite3DB();
-//	authdb->open((char *)"file:mem_authdb?mode=memory&cache=shared", SQLITE_OPEN_READWRITE | SQLITE_OPEN_CREATE | SQLITE_OPEN_FULLMUTEX);
-//	authdb->execute(AUTH_TABLE_MYSQL_USERS);
-//	authdb->execute(AUTH_TABLE_MYSQL_USERS_INCOMING);
-
 };
 
 MySQL_Authentication::~MySQL_Authentication() {
 	reset();
 	delete creds_backends.cred_array;
 	delete creds_frontends.cred_array;
-//	delete authdb;
 };
 
 void MySQL_Authentication::print_version() {
 		fprintf(stderr,"Standard MySQL Authentication rev. %s -- %s -- %s\n", MYSQL_AUTHENTICATION_VERSION, __FILE__, __TIMESTAMP__);
 	};
 
-
-//void MySQL_Authentication::rdlock() {
-//	spin_wrlock(&rwlock);
-//}
-//
-//void MySQL_Authentication::rdunlock() {
-//	spin_wrunlock(&rwlock);
-//}
-//
-//void MySQL_Authentication::wrlock() {
-//	spin_wrlock(&rwlock);
-//}
-//
-//void MySQL_Authentication::wrunlock() {
-//	spin_wrunlock(&rwlock);
-//}
-
-
-//bool MySQL_Authentication::user_add(unsigned int hid, char *add, uint16_t p, unsigned int _weight, enum MySerStatus status, unsigned int _comp /*, uint8_t _charset */, unsigned int _max_connections) {
-//  bool ret;
-//  proxy_debug(PROXY_DEBUG_MYSQL_CONNPOOL, 7, "Adding in mysql_servers_incoming server %s:%d in hostgroup %u with weight %u , status %u, %s compression, max_connections %d\n", add,p,hid,_weight,status, (_comp ? "with" : "without") /*, _charset */ , _max_connections);
-//  char *q=(char *)"INSERT INTO mysql_servers_incoming VALUES (%u, \"%s\", %u, %u, %u, %u, %u)";
-//  char *query=(char *)malloc(strlen(q)+strlen(add)+100);
-//  sprintf(query,q,hid,add,p,_weight,status,_comp /*,_charset */, _max_connections);
-//  ret=mydb->execute(query);
-//  free(query);
-//  return ret;
-//}
-
 void MySQL_Authentication::set_all_inactive(enum cred_username_type usertype) {
 	creds_group_t &cg=(usertype==USERNAME_BACKEND ? creds_backends : creds_frontends);
+#ifdef PROXYSQL_AUTH_PTHREAD_MUTEX
+	pthread_rwlock_wrlock(&cg.lock);
+#else
 	spin_wrlock(&cg.lock);
+#endif
 	unsigned int i;
 	for (i=0; i<cg.cred_array->len; i++) {
 		account_details_t *ado=(account_details_t *)cg.cred_array->index(i);
 		ado->__active=false;
 	}
+#ifdef PROXYSQL_AUTH_PTHREAD_MUTEX
+	pthread_rwlock_unlock(&cg.lock);
+#else
 	spin_wrunlock(&cg.lock);
+#endif
 }
 
 void MySQL_Authentication::remove_inactives(enum cred_username_type usertype) {
 	creds_group_t &cg=(usertype==USERNAME_BACKEND ? creds_backends : creds_frontends);
+#ifdef PROXYSQL_AUTH_PTHREAD_MUTEX
+	pthread_rwlock_wrlock(&cg.lock);
+#else
 	spin_wrlock(&cg.lock);
+#endif
 	unsigned int i;
 __loop_remove_inactives:
 	for (i=0; i<cg.cred_array->len; i++) {
@@ -120,120 +69,167 @@ __loop_remove_inactives:
 			goto __loop_remove_inactives; // we aren't sure how the underlying structure changes, so we jump back to 0
 		}
 	}
+#ifdef PROXYSQL_AUTH_PTHREAD_MUTEX
+	pthread_rwlock_unlock(&cg.lock);
+#else
 	spin_wrunlock(&cg.lock);
+#endif
 }
 
 bool MySQL_Authentication::add(char * username, char * password, enum cred_username_type usertype, bool use_ssl, int default_hostgroup, char *default_schema, bool schema_locked, bool transaction_persistent, bool fast_forward, int max_connections) {
 	uint64_t hash1, hash2;
-	SpookyHash *myhash=new SpookyHash();
-	myhash->Init(1,2);
-	myhash->Update(username,strlen(username));
-	myhash->Final(&hash1,&hash2);
-	delete myhash;
+	SpookyHash myhash;
+	myhash.Init(1,2);
+	myhash.Update(username,strlen(username));
+	myhash.Final(&hash1,&hash2);
 
 	creds_group_t &cg=(usertype==USERNAME_BACKEND ? creds_backends : creds_frontends);
 	
-	void *sha1_pass=NULL;
-	char *oldpass=NULL;
+#ifdef PROXYSQL_AUTH_PTHREAD_MUTEX
+	pthread_rwlock_wrlock(&cg.lock);
+#else
 	spin_wrlock(&cg.lock);
-	//btree::btree_map<uint64_t, account_details_t *>::iterator lookup;
-	std::unordered_map<uint64_t, account_details_t *>::iterator lookup;
+#endif
+	std::map<uint64_t, account_details_t *>::iterator lookup;
 	lookup = cg.bt_map.find(hash1);
-	int current_num_connections_used=0;
+	// few changes will follow, due to issue #802
+	account_details_t *ad=NULL;
+	bool new_ad=false;
 	if (lookup != cg.bt_map.end()) {
-		account_details_t *ad=lookup->second;
-		cg.cred_array->remove_fast(ad);
-     cg.bt_map.erase(lookup);
-		free(ad->username);
-		if (ad->sha1_pass) {
-			oldpass=strdup(ad->password);
-			sha1_pass=malloc(SHA_DIGEST_LENGTH);
-			memcpy(sha1_pass,ad->sha1_pass,SHA_DIGEST_LENGTH);
-		}
-		free(ad->password);
-		if (ad->sha1_pass) {
-			free(ad->sha1_pass);
-			ad->sha1_pass=NULL;
-		}
-		free(ad->default_schema);
-		current_num_connections_used=ad->num_connections_used;
-		free(ad);
-   }
-	account_details_t *ad=(account_details_t *)malloc(sizeof(account_details_t));
-	ad->username=strdup(username);
-	ad->password=strdup(password);
-	ad->sha1_pass=NULL;
-	if (strlen(password)) {
-		if (password[0]=='*') { // password is sha1(sha1(real_password))
-			if (oldpass) {
-				if (strcmp(password,oldpass)==0) { // pass is unchanged
-					ad->sha1_pass=malloc(SHA_DIGEST_LENGTH);
-					memcpy(ad->sha1_pass,sha1_pass,SHA_DIGEST_LENGTH);
-				}
+		ad=lookup->second;
+		if (strcmp(ad->password,password)) {
+			free(ad->password);
+			ad->password=strdup(password);
+			if (ad->sha1_pass) {
+				free(ad->sha1_pass);
+				ad->sha1_pass=NULL;
 			}
 		}
+		if (strcmp(ad->default_schema,default_schema)) {
+			free(ad->default_schema);
+			ad->default_schema=strdup(default_schema);
+		}
+  } else {
+		ad=(account_details_t *)malloc(sizeof(account_details_t));
+		ad->username=strdup(username);
+		ad->default_schema=strdup(default_schema);
+		ad->password=strdup(password);
+		new_ad=true;
+		ad->sha1_pass=NULL;
+		ad->num_connections_used=0;
 	}
+
 	ad->use_ssl=use_ssl;
 	ad->default_hostgroup=default_hostgroup;
-	ad->default_schema=strdup(default_schema);
 	ad->schema_locked=schema_locked;
 	ad->transaction_persistent=transaction_persistent;
 	ad->fast_forward=fast_forward;
 	ad->max_connections=max_connections;
-	ad->num_connections_used=current_num_connections_used;
 	ad->__active=true;
-	cg.bt_map.insert(std::make_pair(hash1,ad));
-	cg.cred_array->add(ad);
+	if (new_ad) {
+		cg.bt_map.insert(std::make_pair(hash1,ad));
+		cg.cred_array->add(ad);
+	}
+#ifdef PROXYSQL_AUTH_PTHREAD_MUTEX
+	pthread_rwlock_unlock(&cg.lock);
+#else
 	spin_wrunlock(&cg.lock);
-
-	if (oldpass) {
-		free(oldpass);
-		oldpass=NULL;
-	}
-	if (sha1_pass) {
-		free(sha1_pass);
-		sha1_pass=NULL;
-	}
+#endif
 	return true;
 };
 
-int MySQL_Authentication::dump_all_users(account_details_t ***ads) {
+
+unsigned int MySQL_Authentication::memory_usage() {
+	unsigned int ret=0;
+#ifdef PROXYSQL_AUTH_PTHREAD_MUTEX
+	pthread_rwlock_rdlock(&creds_frontends.lock);
+	pthread_rwlock_rdlock(&creds_backends.lock);
+#else
 	spin_rdlock(&creds_frontends.lock);
 	spin_rdlock(&creds_backends.lock);
+#endif
+	unsigned i=0;
+	for (i=0; i<creds_frontends.cred_array->len; i++) {
+		account_details_t *ado=(account_details_t *)creds_frontends.cred_array->index(i);
+		ret += sizeof(account_details_t);
+		if (ado->username) ret += strlen(ado->username) + 1;
+		if (ado->password) ret += strlen(ado->password) + 1;
+		if (ado->sha1_pass) ret += SHA_DIGEST_LENGTH;
+		if (ado->default_schema) ret += strlen(ado->default_schema) + 1;
+	}
+	ret += sizeof(creds_group_t);
+	ret += sizeof(PtrArray);
+	ret += (creds_frontends.cred_array->size * sizeof(void *));
+	for (i=0; i<creds_backends.cred_array->len; i++) {
+		account_details_t *ado=(account_details_t *)creds_backends.cred_array->index(i);
+		ret += sizeof(account_details_t);
+		if (ado->username) ret += strlen(ado->username) + 1;
+		if (ado->password) ret += strlen(ado->password) + 1;
+		if (ado->sha1_pass) ret += SHA_DIGEST_LENGTH;
+		if (ado->default_schema) ret += strlen(ado->default_schema) + 1;
+	}
+	ret += sizeof(creds_group_t);
+	ret += sizeof(PtrArray);
+	ret += (creds_backends.cred_array->size * sizeof(void *));
+#ifdef PROXYSQL_AUTH_PTHREAD_MUTEX
+	pthread_rwlock_unlock(&creds_frontends.lock);
+	pthread_rwlock_unlock(&creds_backends.lock);
+#else
+	spin_rdunlock(&creds_frontends.lock);
+	spin_rdunlock(&creds_backends.lock);
+#endif
+	return ret;
+}
+
+int MySQL_Authentication::dump_all_users(account_details_t ***ads, bool _complete) {
+#ifdef PROXYSQL_AUTH_PTHREAD_MUTEX
+	pthread_rwlock_rdlock(&creds_frontends.lock);
+	pthread_rwlock_rdlock(&creds_backends.lock);
+#else
+	spin_rdlock(&creds_frontends.lock);
+	spin_rdlock(&creds_backends.lock);
+#endif
 	int total_size;
 	int idx_=0;
 	unsigned i=0;
 	account_details_t **_ads;
-	//total_size=creds_frontends.bt_map.size()+creds_backends.bt_map.size();
-	total_size=creds_frontends.cred_array->len+creds_backends.cred_array->len;
+	total_size=creds_frontends.cred_array->len;
+	if (_complete) {
+		total_size+=creds_backends.cred_array->len;
+	}
 	if (!total_size) goto __exit_dump_all_users;
 	_ads=(account_details_t **)malloc(sizeof(account_details_t *)*total_size);
-//	btree::btree_map<uint64_t, account_details_t *>::iterator it;
-//	for (it=creds_frontends.bt_map.begin(); it!=creds_frontends.bt_map.end(); it++) {
-//		account_details_t *ad=lookup->second;
-//		ads[idx]=(account_details_t *)malloc(sizeof(account_details_t));	
-//	}
 	for (i=0; i<creds_frontends.cred_array->len; i++) {
 		account_details_t *ad=(account_details_t *)malloc(sizeof(account_details_t));
 		account_details_t *ado=(account_details_t *)creds_frontends.cred_array->index(i);
 		ad->username=strdup(ado->username);
-		ad->password=strdup(ado->password);
-		ad->sha1_pass=NULL;
-		ad->use_ssl=ado->use_ssl;
-		ad->default_hostgroup=ado->default_hostgroup;
-		ad->default_schema=strdup(ado->default_schema);
-		ad->schema_locked=ado->schema_locked;
-		ad->transaction_persistent=ado->transaction_persistent;
-		ad->fast_forward=ado->fast_forward;
 		ad->max_connections=ado->max_connections;
-		ad->__frontend=1;
-		ad->__backend=0;
+		ad->default_hostgroup=ado->default_hostgroup;
+		if (_complete==false) {
+			ad->password=NULL;
+			ad->default_schema=NULL;
+			ad->num_connections_used=ado->num_connections_used;
+		} else {
+			ad->num_connections_used=ado->num_connections_used;
+			ad->password=strdup(ado->password);
+			ad->default_schema=strdup(ado->default_schema);
+			ad->sha1_pass=NULL;
+			ad->use_ssl=ado->use_ssl;
+			ad->default_schema=strdup(ado->default_schema);
+			ad->schema_locked=ado->schema_locked;
+			ad->transaction_persistent=ado->transaction_persistent;
+			ad->fast_forward=ado->fast_forward;
+			ad->__frontend=1;
+			ad->__backend=0;
+		}
 		_ads[idx_]=ad;
 		idx_++;
 	}
+	if (_complete==true) {
 	for (i=0; i<creds_backends.cred_array->len; i++) {
 		account_details_t *ad=(account_details_t *)malloc(sizeof(account_details_t));
 		account_details_t *ado=(account_details_t *)creds_backends.cred_array->index(i);
+		ad->num_connections_used=0;
 		ad->username=strdup(ado->username);
 		ad->password=strdup(ado->password);
 		ad->sha1_pass=NULL;
@@ -249,15 +245,21 @@ int MySQL_Authentication::dump_all_users(account_details_t ***ads) {
 		_ads[idx_]=ad;
 		idx_++;
 	}
+	}
 	*ads=_ads;
 __exit_dump_all_users:
+#ifdef PROXYSQL_AUTH_PTHREAD_MUTEX
+	pthread_rwlock_unlock(&creds_frontends.lock);
+	pthread_rwlock_unlock(&creds_backends.lock);
+#else
 	spin_rdunlock(&creds_frontends.lock);
 	spin_rdunlock(&creds_backends.lock);
+#endif
 	return total_size;
 }
 
 
-int MySQL_Authentication::increase_frontend_user_connections(char *username) {
+int MySQL_Authentication::increase_frontend_user_connections(char *username, int *mc) {
 	uint64_t hash1, hash2;
 	SpookyHash *myhash=new SpookyHash();
 	myhash->Init(1,2);
@@ -266,9 +268,12 @@ int MySQL_Authentication::increase_frontend_user_connections(char *username) {
 	delete myhash;
 	creds_group_t &cg=creds_frontends;
 	int ret=0;
+#ifdef PROXYSQL_AUTH_PTHREAD_MUTEX
+	pthread_rwlock_wrlock(&cg.lock);
+#else
 	spin_wrlock(&cg.lock);
-	//btree::btree_map<uint64_t, account_details_t *>::iterator it;
-	std::unordered_map<uint64_t, account_details_t *>::iterator it;
+#endif
+	std::map<uint64_t, account_details_t *>::iterator it;
 	it = cg.bt_map.find(hash1);
 	if (it != cg.bt_map.end()) {
 		account_details_t *ad=it->second;
@@ -276,8 +281,15 @@ int MySQL_Authentication::increase_frontend_user_connections(char *username) {
 			ret=ad->max_connections-ad->num_connections_used;
 			ad->num_connections_used++;
 		}
+		if (mc) {
+			*mc=ad->max_connections;
+		}
 	}
+#ifdef PROXYSQL_AUTH_PTHREAD_MUTEX
+	pthread_rwlock_unlock(&cg.lock);
+#else
 	spin_wrunlock(&cg.lock);
+#endif
 	return ret;
 }
 
@@ -289,9 +301,12 @@ void MySQL_Authentication::decrease_frontend_user_connections(char *username) {
 	myhash->Final(&hash1,&hash2);
 	delete myhash;
 	creds_group_t &cg=creds_frontends;
+#ifdef PROXYSQL_AUTH_PTHREAD_MUTEX
+	pthread_rwlock_wrlock(&cg.lock);
+#else
 	spin_wrlock(&cg.lock);
-	//btree::btree_map<uint64_t, account_details_t *>::iterator it;
-	std::unordered_map<uint64_t, account_details_t *>::iterator it;
+#endif
+	std::map<uint64_t, account_details_t *>::iterator it;
 	it = cg.bt_map.find(hash1);
 	if (it != cg.bt_map.end()) {
 		account_details_t *ad=it->second;
@@ -299,7 +314,11 @@ void MySQL_Authentication::decrease_frontend_user_connections(char *username) {
 			ad->num_connections_used--;
 		}
 	}
+#ifdef PROXYSQL_AUTH_PTHREAD_MUTEX
+	pthread_rwlock_unlock(&cg.lock);
+#else
 	spin_wrunlock(&cg.lock);
+#endif
 }
 
 bool MySQL_Authentication::del(char * username, enum cred_username_type usertype, bool set_lock) {
@@ -314,9 +333,12 @@ bool MySQL_Authentication::del(char * username, enum cred_username_type usertype
 	creds_group_t &cg=(usertype==USERNAME_BACKEND ? creds_backends : creds_frontends);
 
 	if (set_lock)
+#ifdef PROXYSQL_AUTH_PTHREAD_MUTEX
+		pthread_rwlock_wrlock(&cg.lock);
+#else
 		spin_wrlock(&cg.lock);
-	//btree::btree_map<uint64_t, account_details_t *>::iterator lookup;
-	std::unordered_map<uint64_t, account_details_t *>::iterator lookup;
+#endif
+	std::map<uint64_t, account_details_t *>::iterator lookup;
 	lookup = cg.bt_map.find(hash1);
 	if (lookup != cg.bt_map.end()) {
 		account_details_t *ad=lookup->second;
@@ -330,8 +352,11 @@ bool MySQL_Authentication::del(char * username, enum cred_username_type usertype
 		ret=true;
 	}
 	if (set_lock)
+#ifdef PROXYSQL_AUTH_PTHREAD_MUTEX
+		pthread_rwlock_unlock(&cg.lock);
+#else
 		spin_wrunlock(&cg.lock);
-
+#endif
 	return ret;
 };
 
@@ -346,9 +371,12 @@ bool MySQL_Authentication::set_SHA1(char * username, enum cred_username_type use
 
 	creds_group_t &cg=(usertype==USERNAME_BACKEND ? creds_backends : creds_frontends);
 
+#ifdef PROXYSQL_AUTH_PTHREAD_MUTEX
+	pthread_rwlock_wrlock(&cg.lock);
+#else
 	spin_wrlock(&cg.lock);
-	//btree::btree_map<uint64_t, account_details_t *>::iterator lookup;
-	std::unordered_map<uint64_t, account_details_t *>::iterator lookup;
+#endif
+	std::map<uint64_t, account_details_t *>::iterator lookup;
 	lookup = cg.bt_map.find(hash1);
 	if (lookup != cg.bt_map.end()) {
 		account_details_t *ad=lookup->second;
@@ -359,13 +387,14 @@ bool MySQL_Authentication::set_SHA1(char * username, enum cred_username_type use
 		}
 		ret=true;
 	}
+#ifdef PROXYSQL_AUTH_PTHREAD_MUTEX
+	pthread_rwlock_unlock(&cg.lock);
+#else
    spin_wrunlock(&cg.lock);
-
+#endif
 	return ret;
 };
 
-
-
 char * MySQL_Authentication::lookup(char * username, enum cred_username_type usertype, bool *use_ssl, int *default_hostgroup, char **default_schema, bool *schema_locked, bool *transaction_persistent, bool *fast_forward, int *max_connections, void **sha1_pass) {
 	char *ret=NULL;
 	uint64_t hash1, hash2;
@@ -376,9 +405,12 @@ char * MySQL_Authentication::lookup(char * username, enum cred_username_type use
 
 	creds_group_t &cg=(usertype==USERNAME_BACKEND ? creds_backends : creds_frontends);
 
+#ifdef PROXYSQL_AUTH_PTHREAD_MUTEX
+	pthread_rwlock_rdlock(&cg.lock);
+#else
 	spin_rdlock(&cg.lock);
-	//btree::btree_map<uint64_t, account_details_t *>::iterator lookup;
-	std::unordered_map<uint64_t, account_details_t *>::iterator lookup;
+#endif
+	std::map<uint64_t, account_details_t *>::iterator lookup;
 	lookup = cg.bt_map.find(hash1);
 	if (lookup != cg.bt_map.end()) {
 		account_details_t *ad=lookup->second;
@@ -397,7 +429,11 @@ char * MySQL_Authentication::lookup(char * username, enum cred_username_type use
 			}
 		}
 	}
+#ifdef PROXYSQL_AUTH_PTHREAD_MUTEX
+	pthread_rwlock_unlock(&cg.lock);
+#else
 	spin_rdunlock(&cg.lock);
+#endif
 	return ret;
 
 }
@@ -405,24 +441,33 @@ char * MySQL_Authentication::lookup(char * username, enum cred_username_type use
 bool MySQL_Authentication::_reset(enum cred_username_type usertype) {
 	creds_group_t &cg=(usertype==USERNAME_BACKEND ? creds_backends : creds_frontends);
 
+#ifdef PROXYSQL_AUTH_PTHREAD_MUTEX
+	pthread_rwlock_wrlock(&cg.lock);
+#else
 	spin_wrlock(&cg.lock);
-	//btree::btree_map<uint64_t, account_details_t *>::iterator lookup;
-	std::unordered_map<uint64_t, account_details_t *>::iterator lookup;
+#endif
+	std::map<uint64_t, account_details_t *>::iterator lookup;
 
 	while (cg.bt_map.size()) {
 		lookup = cg.bt_map.begin();
 		if ( lookup != cg.bt_map.end() ) {
 			account_details_t *ad=lookup->second;
-			cg.cred_array->remove_fast(ad);
      	cg.bt_map.erase(lookup);
 			free(ad->username);
 			free(ad->password);
 			if (ad->sha1_pass) { free(ad->sha1_pass); ad->sha1_pass=NULL; }
+			free(ad->default_schema);
 			free(ad);
 		}
 	}
+	while (cg.cred_array->len) {
+		cg.cred_array->remove_index_fast(0);
+	}
+#ifdef PROXYSQL_AUTH_PTHREAD_MUTEX
+	pthread_rwlock_unlock(&cg.lock);
+#else
 	spin_wrunlock(&cg.lock);
-
+#endif
 	return true;
 };
 
@@ -431,3 +476,42 @@ bool MySQL_Authentication::reset() {
 	_reset(USERNAME_FRONTEND);
 	return true;
 }
+
+
+uint64_t MySQL_Authentication::_get_runtime_checksum(enum cred_username_type usertype) {
+	creds_group_t &cg=(usertype==USERNAME_BACKEND ? creds_backends : creds_frontends);
+	std::map<uint64_t, account_details_t *>::iterator it;
+	if (cg.bt_map.size() == 0) {
+		return 0;
+	}
+	bool foundany = false;
+	SpookyHash myhash;
+	myhash.Init(13,4);
+	for (it = cg.bt_map.begin(); it != cg.bt_map.end(); ) {
+		account_details_t *ad=it->second;
+		if (ad->default_hostgroup >= 0) {
+			foundany = true;
+			myhash.Update(&ad->use_ssl,sizeof(ad->use_ssl));
+			myhash.Update(&ad->default_hostgroup,sizeof(ad->default_hostgroup));
+			myhash.Update(&ad->schema_locked,sizeof(ad->schema_locked));
+			myhash.Update(&ad->transaction_persistent,sizeof(ad->transaction_persistent));
+			myhash.Update(&ad->fast_forward,sizeof(ad->fast_forward));
+			myhash.Update(&ad->max_connections,sizeof(ad->max_connections));
+			myhash.Update(ad->username,strlen(ad->username));
+			myhash.Update(ad->password,strlen(ad->password));
+		}
+		it++;
+	}
+	if (foundany == false) {
+		return 0;
+	}
+	uint64_t hash1, hash2;
+	myhash.Final(&hash1, &hash2);
+	return hash1;
+}
+
+uint64_t MySQL_Authentication::get_runtime_checksum() {
+	uint64_t hashB = _get_runtime_checksum(USERNAME_BACKEND);
+	uint64_t hashF = _get_runtime_checksum(USERNAME_FRONTEND);
+	return hashB+hashF;
+}
