@@ -42,6 +42,14 @@ static MySQL_Monitor *GloMyMon;
 	} while (rc!=SQLITE_DONE);\
 } while (0)
 
+#define SAFE_SQLITE3_STEP2(_stmt) do {\
+        do {\
+                rc=sqlite3_step(_stmt);\
+                if (rc==SQLITE_LOCKED || rc==SQLITE_BUSY) {\
+                        usleep(100);\
+                }\
+        } while (rc==SQLITE_LOCKED || rc==SQLITE_BUSY);\
+} while (0)
 
 class ConsumerThread : public Thread {
 	wqueue<WorkItem*>& m_queue;
@@ -387,12 +395,16 @@ void * monitor_connect_thread(void *arg) {
 	rc=sqlite3_bind_int64(statement, 3, time_now); assert(rc==SQLITE_OK);
 	rc=sqlite3_bind_int64(statement, 4, (mmsd->mysql_error_msg ? 0 : mmsd->t2-mmsd->t1)); assert(rc==SQLITE_OK);
 	rc=sqlite3_bind_text(statement, 5, mmsd->mysql_error_msg, -1, SQLITE_TRANSIENT); assert(rc==SQLITE_OK);
-	SAFE_SQLITE3_STEP(statement);
+	SAFE_SQLITE3_STEP2(statement);
 	rc=sqlite3_clear_bindings(statement); assert(rc==SQLITE_OK);
 	rc=sqlite3_reset(statement); assert(rc==SQLITE_OK);
 	sqlite3_finalize(statement);
 	if (mmsd->mysql_error_msg) {
-		if (strncmp(mmsd->mysql_error_msg,"Access denied for user",strlen("Access denied for user"))==0) {
+		if (
+			(strncmp(mmsd->mysql_error_msg,"Access denied for user",strlen("Access denied for user"))==0)
+			||
+			(strncmp(mmsd->mysql_error_msg,"ProxySQL Error: Access denied for user",strlen("ProxySQL Error: Access denied for user"))==0)
+		) {
 			proxy_error("Server %s:%d is returning \"Access denied\" for monitoring user\n", mmsd->hostname, mmsd->port);
 		}
 	}
@@ -466,7 +478,7 @@ __exit_monitor_ping_thread:
 		rc=sqlite3_bind_int64(statement, 3, time_now); assert(rc==SQLITE_OK);
 		rc=sqlite3_bind_int64(statement, 4, (mmsd->mysql_error_msg ? 0 : mmsd->t2-mmsd->t1)); assert(rc==SQLITE_OK);
 		rc=sqlite3_bind_text(statement, 5, mmsd->mysql_error_msg, -1, SQLITE_TRANSIENT); assert(rc==SQLITE_OK);
-		SAFE_SQLITE3_STEP(statement);
+		SAFE_SQLITE3_STEP2(statement);
 		rc=sqlite3_clear_bindings(statement); assert(rc==SQLITE_OK);
 		rc=sqlite3_reset(statement); assert(rc==SQLITE_OK);
 		sqlite3_finalize(statement);
@@ -566,6 +578,7 @@ bool MySQL_Monitor_State_Data::create_new_connection() {
 }
 
 void * monitor_read_only_thread(void *arg) {
+	bool timeout_reached = false;
 	MySQL_Monitor_State_Data *mmsd=(MySQL_Monitor_State_Data *)arg;
 	MySQL_Thread * mysql_thr = new MySQL_Thread();
 	mysql_thr->curtime=monotonic_time();
@@ -584,7 +597,15 @@ void * monitor_read_only_thread(void *arg) {
 		rc=mmsd->create_new_connection();
 		crc=true;
 		if (rc==false) {
-			goto __fast_exit_monitor_read_only_thread;
+			unsigned long long now=monotonic_time();
+			char * new_error = (char *)malloc(50+strlen(mmsd->mysql_error_msg));
+			sprintf(new_error,"timeout on creating new connection: %s",mmsd->mysql_error_msg);
+			free(mmsd->mysql_error_msg);
+			mmsd->mysql_error_msg = new_error;
+			proxy_error("Timeout on read_only check for %s:%d after %lldms. Unable to create a connection. If the server is overload, increase mysql-monitor_connect_timeout. Error: %s.\n", mmsd->hostname, mmsd->port, (now-mmsd->t1)/1000, new_error);
+			timeout_reached = true;
+			goto __exit_monitor_read_only_thread;
+			//goto __fast_exit_monitor_read_only_thread;
 		}
 	}
 
@@ -595,7 +616,8 @@ void * monitor_read_only_thread(void *arg) {
 		unsigned long long now=monotonic_time();
 		if (now > mmsd->t1 + mysql_thread___monitor_read_only_timeout * 1000) {
 			mmsd->mysql_error_msg=strdup("timeout check");
-			proxy_error("Timeout on read_only check for %s:%d after %lldms. If the server is overload, increase mysql-monitor_read_only_timeout. Assuming read_only=1\n", mmsd->hostname, mmsd->port, (now-mmsd->t1)/1000);
+			proxy_error("Timeout on read_only check for %s:%d after %lldms. If the server is overload, increase mysql-monitor_read_only_timeout.\n", mmsd->hostname, mmsd->port, (now-mmsd->t1)/1000);
+			timeout_reached = true;
 			goto __exit_monitor_read_only_thread;
 		}
 		if (GloMyMon->shutdown==true) {
@@ -611,7 +633,8 @@ void * monitor_read_only_thread(void *arg) {
 		unsigned long long now=monotonic_time();
 		if (now > mmsd->t1 + mysql_thread___monitor_read_only_timeout * 1000) {
 			mmsd->mysql_error_msg=strdup("timeout check");
-			proxy_error("Timeout on read_only check for %s:%d after %lldms. If the server is overload, increase mysql-monitor_read_only_timeout. Assuming read_only=1\n", mmsd->hostname, mmsd->port, (now-mmsd->t1)/1000);
+			proxy_error("Timeout on read_only check for %s:%d after %lldms. If the server is overload, increase mysql-monitor_read_only_timeout.\n", mmsd->hostname, mmsd->port, (now-mmsd->t1)/1000);
+			timeout_reached = true;
 			goto __exit_monitor_read_only_thread;
 		}
 		if (GloMyMon->shutdown==true) {
@@ -675,13 +698,40 @@ __exit_monitor_read_only_thread:
 			rc=sqlite3_bind_null(statement, 5); assert(rc==SQLITE_OK);
 		}
 		rc=sqlite3_bind_text(statement, 6, mmsd->mysql_error_msg, -1, SQLITE_TRANSIENT); assert(rc==SQLITE_OK);
-		SAFE_SQLITE3_STEP(statement);
+		SAFE_SQLITE3_STEP2(statement);
 		rc=sqlite3_clear_bindings(statement); assert(rc==SQLITE_OK);
 		rc=sqlite3_reset(statement); assert(rc==SQLITE_OK);
-
-		MyHGM->read_only_action(mmsd->hostname, mmsd->port, read_only);
-
 		sqlite3_finalize(statement);
+
+		if (timeout_reached == false) {
+			MyHGM->read_only_action(mmsd->hostname, mmsd->port, read_only); // default behavior
+		} else {
+			char *error=NULL;
+			int cols=0;
+			int affected_rows=0;
+			SQLite3_result *resultset=NULL;
+			char *new_query=NULL;
+			SQLite3DB *mondb=mmsd->mondb;
+			new_query=(char *)"SELECT 1 FROM (SELECT hostname,port,read_only,error FROM mysql_server_read_only_log WHERE hostname='%s' AND port='%d' ORDER BY time_start_us DESC LIMIT %d) a WHERE read_only IS NULL AND SUBSTR(error,1,7) = 'timeout' GROUP BY hostname,port HAVING COUNT(*)=%d";
+			char *buff=(char *)malloc(strlen(new_query)+strlen(mmsd->hostname)+32);
+			int max_failures=mysql_thread___monitor_read_only_max_timeout_count;
+			sprintf(buff,new_query, mmsd->hostname, mmsd->port, max_failures, max_failures);
+			mondb->execute_statement(buff, &error , &cols , &affected_rows , &resultset);
+			if (!error) {
+				if (resultset) {
+					if (resultset->rows_count) {
+						// disable host
+						proxy_error("Server %s:%d missed %d read_only checks. Assuming read_only=1\n", mmsd->hostname, mmsd->port, max_failures);
+						MyHGM->read_only_action(mmsd->hostname, mmsd->port, read_only); // N timeouts reached
+					}
+					delete resultset;
+					resultset=NULL;
+				}
+			} else {
+				proxy_error("Error on %s : %s\n", buff, error);
+			}
+			free(buff);
+		}
 	}
 	if (mmsd->interr) { // check failed
 	} else {
@@ -916,7 +966,7 @@ __end_process_group_replication_result:
 			rc=sqlite3_bind_null(statement, 5); assert(rc==SQLITE_OK);
 		}
 		rc=sqlite3_bind_text(statement, 6, mmsd->mysql_error_msg, -1, SQLITE_TRANSIENT); assert(rc==SQLITE_OK);
-		SAFE_SQLITE3_STEP(statement);
+		SAFE_SQLITE3_STEP2(statement);
 		rc=sqlite3_clear_bindings(statement); assert(rc==SQLITE_OK);
 		rc=sqlite3_reset(statement); assert(rc==SQLITE_OK);
 
@@ -1089,7 +1139,7 @@ __exit_monitor_replication_lag_thread:
 					rc=sqlite3_bind_null(statement, 5); assert(rc==SQLITE_OK);
 				}
 				rc=sqlite3_bind_text(statement, 6, mmsd->mysql_error_msg, -1, SQLITE_TRANSIENT); assert(rc==SQLITE_OK);
-				SAFE_SQLITE3_STEP(statement);
+				SAFE_SQLITE3_STEP2(statement);
 				rc=sqlite3_clear_bindings(statement); assert(rc==SQLITE_OK);
 				rc=sqlite3_reset(statement); assert(rc==SQLITE_OK);
 				MyHGM->replication_lag_action(mmsd->hostgroup_id, mmsd->hostname, mmsd->port, repl_lag);
@@ -1181,12 +1231,16 @@ void * MySQL_Monitor::monitor_connect() {
 			}
 			for (std::vector<SQLite3_row *>::iterator it = resultset->rows.begin() ; it != resultset->rows.end(); ++it) {
 				SQLite3_row *r=*it;
-				MySQL_Monitor_State_Data *mmsd=new MySQL_Monitor_State_Data(r->fields[0],atoi(r->fields[1]), NULL, atoi(r->fields[2]));
-				mmsd->mondb=monitordb;
-				WorkItem* item;
-				item=new WorkItem(mmsd,monitor_connect_thread);
-				GloMyMon->queue.add(item);
-				usleep(us);
+				bool rc_ping = true;
+				rc_ping = server_responds_to_ping(r->fields[0],atoi(r->fields[1]));
+				if (rc_ping) { // only if server is responding to pings
+					MySQL_Monitor_State_Data *mmsd=new MySQL_Monitor_State_Data(r->fields[0],atoi(r->fields[1]), NULL, atoi(r->fields[2]));
+					mmsd->mondb=monitordb;
+					WorkItem* item;
+					item=new WorkItem(mmsd,monitor_connect_thread);
+					GloMyMon->queue.add(item);
+					usleep(us);
+				}
 				if (GloMyMon->shutdown) return NULL;
 			}
 		}
@@ -1207,7 +1261,7 @@ __end_monitor_connect_loop:
 			}
 			unsigned long long time_now=realtime_time();
 			rc=sqlite3_bind_int64(statement, 1, time_now-(unsigned long long)mysql_thread___monitor_history*1000); assert(rc==SQLITE_OK);
-			SAFE_SQLITE3_STEP(statement);
+			SAFE_SQLITE3_STEP2(statement);
 			rc=sqlite3_clear_bindings(statement); assert(rc==SQLITE_OK);
 			rc=sqlite3_reset(statement); assert(rc==SQLITE_OK);
 			sqlite3_finalize(statement);
@@ -1315,7 +1369,7 @@ __end_monitor_ping_loop:
 			}
 			unsigned long long time_now=realtime_time();
 			rc=sqlite3_bind_int64(statement, 1, time_now-(unsigned long long)mysql_thread___monitor_history*1000); assert(rc==SQLITE_OK);
-			SAFE_SQLITE3_STEP(statement);
+			SAFE_SQLITE3_STEP2(statement);
 			rc=sqlite3_clear_bindings(statement); assert(rc==SQLITE_OK);
 			rc=sqlite3_reset(statement); assert(rc==SQLITE_OK);
 			sqlite3_finalize(statement);
@@ -1349,7 +1403,7 @@ __end_monitor_ping_loop:
 				resultset=NULL;
 			}
 			char *new_query=NULL;
-			new_query=(char *)"SELECT 1 FROM (SELECT hostname,port,ping_error FROM mysql_server_ping_log WHERE hostname='%s' AND port='%s' ORDER BY time_start_us DESC LIMIT %d) a WHERE ping_error IS NOT NULL AND ping_error NOT LIKE 'Access denied for user%%' GROUP BY hostname,port HAVING COUNT(*)=%d";
+			new_query=(char *)"SELECT 1 FROM (SELECT hostname,port,ping_error FROM mysql_server_ping_log WHERE hostname='%s' AND port='%s' ORDER BY time_start_us DESC LIMIT %d) a WHERE ping_error IS NOT NULL AND ping_error NOT LIKE 'Access denied for user%%' AND ping_error NOT LIKE 'ProxySQL Error: Access denied for user%%' GROUP BY hostname,port HAVING COUNT(*)=%d";
 			for (j=0;j<i;j++) {
 				char *buff=(char *)malloc(strlen(new_query)+strlen(addresses[j])+strlen(ports[j])+16);
 				int max_failures=mysql_thread___monitor_ping_max_failures;
@@ -1359,8 +1413,11 @@ __end_monitor_ping_loop:
 					if (resultset) {
 						if (resultset->rows_count) {
 							// disable host
-							proxy_error("Server %s:%s missed %d heartbeats, shunning it and killing all the connections\n", addresses[j], ports[j], max_failures);
-							MyHGM->shun_and_killall(addresses[j],atoi(ports[j]));
+							bool rc_shun = false;
+							rc_shun = MyHGM->shun_and_killall(addresses[j],atoi(ports[j]));
+							if (rc_shun) {
+								proxy_error("Server %s:%s missed %d heartbeats, shunning it and killing all the connections. Disabling other checks until the node comes back online.\n", addresses[j], ports[j], max_failures);
+							}
 						}
 						delete resultset;
 						resultset=NULL;
@@ -1458,6 +1515,39 @@ __sleep_monitor_ping_loop:
 	return NULL;
 }
 
+
+
+bool MySQL_Monitor::server_responds_to_ping(char *address, int port) {
+	bool ret = true; // default
+	char *error=NULL;
+	int cols=0;
+	int affected_rows=0;
+	SQLite3_result *resultset=NULL;
+	char *new_query=NULL;
+	new_query=(char *)"SELECT 1 FROM (SELECT hostname,port,ping_error FROM mysql_server_ping_log WHERE hostname='%s' AND port=%d ORDER BY time_start_us DESC LIMIT %d) a WHERE ping_error IS NOT NULL AND ping_error NOT LIKE 'Access denied for user%%' GROUP BY hostname,port HAVING COUNT(*)=%d";
+	char *buff=(char *)malloc(strlen(new_query)+strlen(address)+32);
+	int max_failures = mysql_thread___monitor_ping_max_failures;
+	sprintf(buff,new_query,address,port,max_failures,max_failures);
+	monitordb->execute_statement(buff, &error , &cols , &affected_rows , &resultset);
+	if (!error) {
+		if (resultset) {
+			if (resultset->rows_count) {
+				ret = false;
+			}
+			delete resultset;
+			resultset=NULL;
+		}
+	} else {
+		proxy_error("Error on %s : %s\n", buff, error);
+	}
+	if (resultset) {
+		delete resultset;
+		resultset=NULL;
+	}
+	free(buff);
+	return ret;
+}
+
 void * MySQL_Monitor::monitor_read_only() {
 	// initialize the MySQL Thread (note: this is not a real thread, just the structures associated with it)
 	unsigned int MySQL_Monitor__thread_MySQL_Thread_Variables_version;
@@ -1508,12 +1598,16 @@ void * MySQL_Monitor::monitor_read_only() {
 			}
 			for (std::vector<SQLite3_row *>::iterator it = resultset->rows.begin() ; it != resultset->rows.end(); ++it) {
 				SQLite3_row *r=*it;
-				MySQL_Monitor_State_Data *mmsd=new MySQL_Monitor_State_Data(r->fields[0],atoi(r->fields[1]), NULL, atoi(r->fields[2]));
-				mmsd->mondb=monitordb;
-				WorkItem* item;
-				item=new WorkItem(mmsd,monitor_read_only_thread);
-				GloMyMon->queue.add(item);
-				usleep(us);
+				bool rc_ping = true;
+				rc_ping = server_responds_to_ping(r->fields[0],atoi(r->fields[1]));
+				if (rc_ping) { // only if server is responding to pings
+					MySQL_Monitor_State_Data *mmsd=new MySQL_Monitor_State_Data(r->fields[0],atoi(r->fields[1]), NULL, atoi(r->fields[2]));
+					mmsd->mondb=monitordb;
+					WorkItem* item;
+					item=new WorkItem(mmsd,monitor_read_only_thread);
+					GloMyMon->queue.add(item);
+					usleep(us);
+				}
 				if (GloMyMon->shutdown) return NULL;
 			}
 		}
@@ -1527,13 +1621,13 @@ __end_monitor_read_only_loop:
 			query=(char *)"DELETE FROM mysql_server_read_only_log WHERE time_start_us < ?1";
 			rc=sqlite3_prepare_v2(mondb, query, -1, &statement, 0);
 			assert(rc==SQLITE_OK);
-			if (mysql_thread___monitor_history < mysql_thread___monitor_ping_interval * (mysql_thread___monitor_ping_max_failures + 1 )) { // issue #626
-				if (mysql_thread___monitor_ping_interval < 3600000)
-					mysql_thread___monitor_history = mysql_thread___monitor_ping_interval * (mysql_thread___monitor_ping_max_failures + 1 );
+			if (mysql_thread___monitor_history < mysql_thread___monitor_read_only_interval * (mysql_thread___monitor_read_only_max_timeout_count + 1 )) { // issue #626
+				if (mysql_thread___monitor_read_only_interval < 3600000)
+					mysql_thread___monitor_history = mysql_thread___monitor_read_only_interval * (mysql_thread___monitor_read_only_max_timeout_count + 1 );
 			}
 			unsigned long long time_now=realtime_time();
 			rc=sqlite3_bind_int64(statement, 1, time_now-(unsigned long long)mysql_thread___monitor_history*1000); assert(rc==SQLITE_OK);
-			SAFE_SQLITE3_STEP(statement);
+			SAFE_SQLITE3_STEP2(statement);
 			rc=sqlite3_clear_bindings(statement); assert(rc==SQLITE_OK);
 			rc=sqlite3_reset(statement); assert(rc==SQLITE_OK);
 			sqlite3_finalize(statement);
@@ -1658,7 +1752,7 @@ __end_monitor_group_replication_loop:
 			}
 			unsigned long long time_now=realtime_time();
 			rc=sqlite3_bind_int64(statement, 1, time_now-(unsigned long long)mysql_thread___monitor_history*1000); assert(rc==SQLITE_OK);
-			SAFE_SQLITE3_STEP(statement);
+			SAFE_SQLITE3_STEP2(statement);
 			rc=sqlite3_clear_bindings(statement); assert(rc==SQLITE_OK);
 			rc=sqlite3_reset(statement); assert(rc==SQLITE_OK);
 			sqlite3_finalize(statement);
@@ -1744,12 +1838,16 @@ void * MySQL_Monitor::monitor_replication_lag() {
 			}
 			for (std::vector<SQLite3_row *>::iterator it = resultset->rows.begin() ; it != resultset->rows.end(); ++it) {
 				SQLite3_row *r=*it;
-				MySQL_Monitor_State_Data *mmsd = new MySQL_Monitor_State_Data(r->fields[1], atoi(r->fields[2]), NULL, atoi(r->fields[4]), atoi(r->fields[0]));
-				mmsd->mondb=monitordb;
-				WorkItem* item;
-				item=new WorkItem(mmsd,monitor_replication_lag_thread);
-				GloMyMon->queue.add(item);
-				usleep(us);
+				bool rc_ping = true;
+				rc_ping = server_responds_to_ping(r->fields[0],atoi(r->fields[1]));
+				if (rc_ping) { // only if server is responding to pings
+					MySQL_Monitor_State_Data *mmsd = new MySQL_Monitor_State_Data(r->fields[1], atoi(r->fields[2]), NULL, atoi(r->fields[4]), atoi(r->fields[0]));
+					mmsd->mondb=monitordb;
+					WorkItem* item;
+					item=new WorkItem(mmsd,monitor_replication_lag_thread);
+					GloMyMon->queue.add(item);
+					usleep(us);
+				}
 				if (GloMyMon->shutdown) return NULL;
 			}
 		}
@@ -1769,7 +1867,7 @@ __end_monitor_replication_lag_loop:
 			}
 			unsigned long long time_now=realtime_time();
 			rc=sqlite3_bind_int64(statement, 1, time_now-(unsigned long long)mysql_thread___monitor_history*1000); assert(rc==SQLITE_OK);
-			SAFE_SQLITE3_STEP(statement);
+			SAFE_SQLITE3_STEP2(statement);
 			rc=sqlite3_clear_bindings(statement); assert(rc==SQLITE_OK);
 			rc=sqlite3_reset(statement); assert(rc==SQLITE_OK);
 			sqlite3_finalize(statement);
@@ -1997,7 +2095,7 @@ void MySQL_Monitor::populate_monitor_mysql_server_group_replication_log() {
 				rc=sqlite3_bind_text(statement1, 6, ( node->last_entries[i].read_only ? (char *)"YES" : (char *)"NO" ) , -1, SQLITE_TRANSIENT); assert(rc==SQLITE_OK);
 				rc=sqlite3_bind_int64(statement1, 7, node->last_entries[i].transactions_behind ); assert(rc==SQLITE_OK);
 				rc=sqlite3_bind_text(statement1, 8, node->last_entries[i].error , -1, SQLITE_TRANSIENT); assert(rc==SQLITE_OK);
-				SAFE_SQLITE3_STEP(statement1);
+				SAFE_SQLITE3_STEP2(statement1);
 				rc=sqlite3_clear_bindings(statement1); assert(rc==SQLITE_OK);
 				rc=sqlite3_reset(statement1); assert(rc==SQLITE_OK);
 			}
