@@ -20,7 +20,14 @@
   } while (rc!=SQLITE_DONE);\
 } while (0)
 
-
+#define SAFE_SQLITE3_STEP2(_stmt) do {\
+        do {\
+                rc=sqlite3_step(_stmt);\
+                if (rc==SQLITE_LOCKED || rc==SQLITE_BUSY) {\
+                        usleep(100);\
+                }\
+        } while (rc==SQLITE_LOCKED || rc==SQLITE_BUSY);\
+} while (0)
 
 static char *NODE_COMPUTE_DELIMITER=(char *)"-gtyw23a-"; // a random string used for hashing
 
@@ -167,6 +174,7 @@ void * ProxySQL_Cluster_Monitor_thread(void *args) {
 				}
 				if (conn->net.vio) { 
 					mysql_close(conn);
+					conn = NULL;
 				}
 			} else {
 				proxy_warning("Cluster: unable to connect to peer %s:%d . Error: %s\n", node->hostname, node->port, mysql_error(conn));
@@ -178,9 +186,10 @@ void * ProxySQL_Cluster_Monitor_thread(void *args) {
 		} else {
 			sleep(1);	// do not monitor if the username is empty
 		}
+		rc_bool = GloProxyCluster->Update_Node_Metrics(node->hostname, node->port, NULL, 0); // added extra check, see #1323 
 	}
 __exit_monitor_thread:
-	//if (conn) {
+	if (conn)
 	if (conn->net.vio) {
 		mysql_close(conn);
 	}
@@ -614,7 +623,7 @@ void ProxySQL_Cluster::pull_mysql_query_rules_from_peer() {
 						rc=sqlite3_bind_text(statement1, 29, row[27], -1, SQLITE_TRANSIENT); assert(rc==SQLITE_OK); // log
 						rc=sqlite3_bind_text(statement1, 30, row[28], -1, SQLITE_TRANSIENT); assert(rc==SQLITE_OK); // apply
 						rc=sqlite3_bind_text(statement1, 31, row[29], -1, SQLITE_TRANSIENT); assert(rc==SQLITE_OK); // comment
-						SAFE_SQLITE3_STEP(statement1);
+						SAFE_SQLITE3_STEP2(statement1);
 						rc=sqlite3_clear_bindings(statement1); assert(rc==SQLITE_OK);
 						rc=sqlite3_reset(statement1); assert(rc==SQLITE_OK);
 					}
@@ -695,7 +704,7 @@ void ProxySQL_Cluster::pull_mysql_users_from_peer() {
 						rc=sqlite3_bind_int64(statement1, 11, atoll(row[10])); assert(rc==SQLITE_OK); // frontend
 						rc=sqlite3_bind_int64(statement1, 12, atoll(row[11])); assert(rc==SQLITE_OK); // max_connection
 
-						SAFE_SQLITE3_STEP(statement1);
+						SAFE_SQLITE3_STEP2(statement1);
 						rc=sqlite3_clear_bindings(statement1); assert(rc==SQLITE_OK);
 						rc=sqlite3_reset(statement1); assert(rc==SQLITE_OK);
 					}
@@ -761,7 +770,7 @@ void ProxySQL_Cluster::pull_mysql_servers_from_peer() {
 					result1 = mysql_store_result(conn);
 
 					//rc_query = mysql_query(conn,"SELECT writer_hostgroup, reader_hostgroup, comment FROM runtime_mysql_replication_hostgroups");
-					rc_query = mysql_query(conn,CLSUTER_QUERY_MYSQL_REPLICATION_HOSTGROUPS);
+					rc_query = mysql_query(conn,CLUSTER_QUERY_MYSQL_REPLICATION_HOSTGROUPS);
 					if ( rc_query == 0 ) {
 						result2 = mysql_store_result(conn);
 						proxy_info("Cluster: Fetching MySQL Servers from peer %s:%d completed\n", hostname, port);
@@ -1112,7 +1121,13 @@ bool ProxySQL_Cluster_Nodes::Update_Node_Metrics(char * _h, uint16_t _p, MYSQL_R
 	std::unordered_map<uint64_t, ProxySQL_Node_Entry *>::iterator ite = umap_proxy_nodes.find(hash_);
 	if (ite != umap_proxy_nodes.end()) {
 		ProxySQL_Node_Entry * node = ite->second;
-		node->set_metrics(_r, _response_time);
+		if (_r) {
+			node->set_metrics(_r, _response_time);
+		} else {
+			// if _r is NULL, this function is being called only to verify if
+			// the node should still be checked or not
+			// see bug #1323
+		}
 		ret = true;
 	}
 	pthread_mutex_unlock(&mutex);
