@@ -49,6 +49,29 @@ MySQL_Connection_userinfo::~MySQL_Connection_userinfo() {
 	if (schemaname) free(schemaname);
 }
 
+void MySQL_Connection::compute_unknown_transaction_status() {
+	if (mysql) {
+		int _myerrno=mysql_errno(mysql);
+		if (_myerrno == 0) {
+			unknown_transaction_status = false; // no error
+			return;
+		}
+		if (_myerrno >= 2000 && _myerrno < 3000) { // client error
+			// do not change it
+			return;
+		}
+		if (_myerrno >= 1000 && _myerrno < 2000) { // server error
+			unknown_transaction_status = true;
+			return;
+		}
+		if (_myerrno >= 3000 && _myerrno < 4000) { // server error
+			unknown_transaction_status = true;
+			return;
+		}
+		// all other cases, server error
+	}
+}
+
 uint64_t MySQL_Connection_userinfo::compute_hash() {
 	int l=0;
 	if (username)
@@ -188,14 +211,11 @@ MySQL_Connection::MySQL_Connection() {
 	largest_query_length=0;
 	multiplex_delayed=false;
 	MyRS=NULL;
+	unknown_transaction_status = false;
 	creation_time=0;
 	processing_multi_statement=false;
 	proxy_debug(PROXY_DEBUG_MYSQL_CONNPOOL, 4, "Creating new MySQL_Connection %p\n", this);
-#ifndef PROXYSQL_STMT_V14
-	local_stmts=new MySQL_STMTs_local(false); // false by default, it is a backend
-#else
 	local_stmts=new MySQL_STMTs_local_v14(false); // false by default, it is a backend
-#endif
 };
 
 MySQL_Connection::~MySQL_Connection() {
@@ -918,12 +938,14 @@ handler_again:
 			break;
 		case ASYNC_USE_RESULT_CONT:
 			{
-				unsigned int buffered_data=0;
-				buffered_data = myds->sess->client_myds->PSarrayOUT->len * RESULTSET_BUFLEN;
-				buffered_data += myds->sess->client_myds->resultset->len * RESULTSET_BUFLEN;
-				if (buffered_data > (unsigned int)mysql_thread___threshold_resultset_size*8) {
-					next_event(ASYNC_USE_RESULT_CONT); // we temporarily pause . See #1232
-					break;
+				if (myds->sess && myds->sess->client_myds && myds->sess->mirror==false) {
+					unsigned int buffered_data=0;
+					buffered_data = myds->sess->client_myds->PSarrayOUT->len * RESULTSET_BUFLEN;
+					buffered_data += myds->sess->client_myds->resultset->len * RESULTSET_BUFLEN;
+					if (buffered_data > (unsigned int)mysql_thread___threshold_resultset_size*8) {
+						next_event(ASYNC_USE_RESULT_CONT); // we temporarily pause . See #1232
+						break;
+					}
 				}
 			}
 			if (async_fetch_row_start==false) {
@@ -967,6 +989,14 @@ handler_again:
 			}
 			break;
 		case ASYNC_QUERY_END:
+			if (mysql) {
+				int _myerrno=mysql_errno(mysql);
+				if (_myerrno == 0) {
+					unknown_transaction_status = false;
+				} else {
+					compute_unknown_transaction_status();
+				}
+			}
 			if (mysql_result) {
 				mysql_free_result(mysql_result);
 				mysql_result=NULL;
@@ -1123,14 +1153,16 @@ int MySQL_Connection::async_connect(short event) {
 		return 0;
 	}
 	if (async_state_machine==ASYNC_CONNECT_SUCCESSFUL) {
+		compute_unknown_transaction_status();
 		async_state_machine=ASYNC_IDLE;
 		myds->wait_until=0;
-		creation_time=myds->sess->thread->curtime;
+		creation_time = monotonic_time();
 		return 0;
 	}
 	handler(event);
 	switch (async_state_machine) {
 		case ASYNC_CONNECT_SUCCESSFUL:
+			compute_unknown_transaction_status();
 			async_state_machine=ASYNC_IDLE;
 			myds->wait_until=0;
 			return 0;
@@ -1209,6 +1241,7 @@ int MySQL_Connection::async_query(short event, char *stmt, unsigned long length,
 	}
 	
 	if (async_state_machine==ASYNC_QUERY_END) {
+		compute_unknown_transaction_status();
 		if (mysql_errno(mysql)) {
 			return -1;
 		} else {
@@ -1218,6 +1251,7 @@ int MySQL_Connection::async_query(short event, char *stmt, unsigned long length,
 	if (async_state_machine==ASYNC_STMT_EXECUTE_END) {
 		query.stmt_meta=NULL;
 		async_state_machine=ASYNC_QUERY_END;
+		compute_unknown_transaction_status();
 		if (mysql_stmt_errno(query.stmt)) {
 			return -1;
 		} else {
@@ -1226,6 +1260,7 @@ int MySQL_Connection::async_query(short event, char *stmt, unsigned long length,
 	}
 	if (async_state_machine==ASYNC_STMT_PREPARE_SUCCESSFUL || async_state_machine==ASYNC_STMT_PREPARE_FAILED) {
 		query.stmt_meta=NULL;
+		compute_unknown_transaction_status();
 		if (async_state_machine==ASYNC_STMT_PREPARE_FAILED) {
 			return -1;
 		} else {
@@ -1258,6 +1293,7 @@ int MySQL_Connection::async_ping(short event) {
 	assert(ret_mysql);
 	switch (async_state_machine) {
 		case ASYNC_PING_SUCCESSFUL:
+			unknown_transaction_status = false;
 			async_state_machine=ASYNC_IDLE;
 			return 0;
 			break;
@@ -1277,6 +1313,7 @@ int MySQL_Connection::async_ping(short event) {
 	// check again
 	switch (async_state_machine) {
 		case ASYNC_PING_SUCCESSFUL:
+			unknown_transaction_status = false;
 			async_state_machine=ASYNC_IDLE;
 			return 0;
 			break;
@@ -1299,6 +1336,7 @@ int MySQL_Connection::async_change_user(short event) {
 	assert(ret_mysql);
 	switch (async_state_machine) {
 		case ASYNC_CHANGE_USER_SUCCESSFUL:
+			unknown_transaction_status = false;
 			async_state_machine=ASYNC_IDLE;
 			return 0;
 			break;
@@ -1315,6 +1353,7 @@ int MySQL_Connection::async_change_user(short event) {
 	// check again
 	switch (async_state_machine) {
 		case ASYNC_CHANGE_USER_SUCCESSFUL:
+			unknown_transaction_status = false;
 			async_state_machine=ASYNC_IDLE;
 			return 0;
 			break;
@@ -1334,6 +1373,7 @@ int MySQL_Connection::async_select_db(short event) {
 	assert(ret_mysql);
 	switch (async_state_machine) {
 		case ASYNC_INITDB_SUCCESSFUL:
+			unknown_transaction_status = false;
 			async_state_machine=ASYNC_IDLE;
 			return 0;
 			break;
@@ -1350,6 +1390,7 @@ int MySQL_Connection::async_select_db(short event) {
 	// check again
 	switch (async_state_machine) {
 		case ASYNC_INITDB_SUCCESSFUL:
+			unknown_transaction_status = false;
 			async_state_machine=ASYNC_IDLE;
 			return 0;
 			break;
@@ -1369,6 +1410,7 @@ int MySQL_Connection::async_set_autocommit(short event, bool ac) {
 	assert(ret_mysql);
 	switch (async_state_machine) {
 		case ASYNC_SET_AUTOCOMMIT_SUCCESSFUL:
+			unknown_transaction_status = false;
 			async_state_machine=ASYNC_IDLE;
 			return 0;
 			break;
@@ -1386,6 +1428,7 @@ int MySQL_Connection::async_set_autocommit(short event, bool ac) {
 	// check again
 	switch (async_state_machine) {
 		case ASYNC_SET_AUTOCOMMIT_SUCCESSFUL:
+			unknown_transaction_status = false;
 			async_state_machine=ASYNC_IDLE;
 			return 0;
 			break;
@@ -1405,6 +1448,7 @@ int MySQL_Connection::async_set_names(short event, uint8_t c) {
 	assert(ret_mysql);
 	switch (async_state_machine) {
 		case ASYNC_SET_NAMES_SUCCESSFUL:
+			unknown_transaction_status = false;
 			async_state_machine=ASYNC_IDLE;
 			return 0;
 			break;
@@ -1422,6 +1466,7 @@ int MySQL_Connection::async_set_names(short event, uint8_t c) {
 	// check again
 	switch (async_state_machine) {
 		case ASYNC_SET_NAMES_SUCCESSFUL:
+			unknown_transaction_status = false;
 			async_state_machine=ASYNC_IDLE;
 			return 0;
 			break;
@@ -1466,6 +1511,7 @@ void MySQL_Connection::async_free_result() {
 			mysql_result=NULL;
 		}
 	}
+	compute_unknown_transaction_status();
 	async_state_machine=ASYNC_IDLE;
 	if (MyRS) {
 		delete MyRS;
@@ -1478,7 +1524,7 @@ bool MySQL_Connection::IsActiveTransaction() {
 	bool ret=false;
 	if (mysql) {
 		ret = (mysql->server_status & SERVER_STATUS_IN_TRANS);
-		if (ret == false && (mysql)->net.last_errno) {
+		if (ret == false && (mysql)->net.last_errno && unknown_transaction_status == true) {
 			ret = true;
 		}
 		if (ret == false) {
@@ -1549,7 +1595,7 @@ void MySQL_Connection::ProcessQueryAndSetStatusFlags(char *query_digest_text) {
 		}
 		if (strncasecmp(query_digest_text,"SET ",4)==0) {
 			// For issue #555 , multiplexing is disabled if --safe-updates is used (see session_vars definition)
-			for (unsigned int i = 0; i < sizeof(session_vars); i++) {
+			for (unsigned int i = 0; i < sizeof(session_vars)/sizeof(char *); i++) {
 				if (strcasestr(query_digest_text,session_vars[i])!=NULL)  {
 					set_status_user_variable(true);
 					break;
@@ -1663,6 +1709,7 @@ int MySQL_Connection::async_send_simple_command(short event, char *stmt, unsigne
 		proxy_error("Retrieved a resultset while running a simple command. This is an error!! Simple command: %s\n", stmt);
 	}
 	if (async_state_machine==ASYNC_QUERY_END) {
+		compute_unknown_transaction_status();
 		if (mysql_errno(mysql)) {
 			return -1;
 		} else {
@@ -1700,9 +1747,6 @@ void MySQL_Connection::reset() {
 		}
 	}
 	delete local_stmts;
-#ifndef PROXYSQL_STMT_V14
-	local_stmts=new MySQL_STMTs_local(false);
-#else
 	local_stmts=new MySQL_STMTs_local_v14(false);
-#endif
+	creation_time = monotonic_time();
 }
