@@ -342,6 +342,15 @@ int MySQL_Data_Stream::write_to_net() {
 		if (mypolls) mypolls->last_sent[poll_fds_idx]=sess->thread->curtime;
 		bytes_info.bytes_sent+=bytes_io;
 	}
+	if (bytes_io > 0) {
+		if (myds_type == MYDS_FRONTEND) {
+			if (sess) {
+				if (sess->thread) {
+					sess->thread->status_variables.queries_frontends_bytes_sent += bytes_io;
+				}
+			}
+		}
+	}
 	return bytes_io;
 }
 
@@ -474,9 +483,37 @@ int MySQL_Data_Stream::buffer2array() {
 			if (payload_length) {
 				// the payload is compressed
 				destLen=payload_length;
-				dest=(Bytef *)l_alloc(destLen);
+				//dest=(Bytef *)l_alloc(destLen);
+				dest=(Bytef *)malloc(destLen);
 				int rc=uncompress(dest, &destLen, _ptr, queueIN.pkt.size-7);
-				assert(rc==Z_OK); 
+				if (rc!=Z_OK) {
+					// for some reason, uncompress failed
+					// accoding to debugging on #1410 , it seems some library may send uncompress data claiming it is compressed
+					// we try to assume it is not compressed, and we do some sanity check
+					memcpy(dest, _ptr, queueIN.pkt.size-7);
+					datalength=queueIN.pkt.size-7;
+					// some sanity check now
+					unsigned char _u;
+					bool sanity_check = false;
+					_u = *(u+9);
+					// 2nd and 3rd bytes are 0
+					if (_u == 0) {
+						_u = *(u+8);
+						if (_u == 0) {
+							_u = *(u+7);
+							// 1st byte = size - 7
+							unsigned int _size = _u ;
+							if (queueIN.pkt.size-7 == _size) {
+								sanity_check = true;
+							}
+						}
+					}
+					if (sanity_check == false) {
+						proxy_error("Unable to uncompress a compressed packet\n");
+						shut_soft();
+						return ret;
+					}
+				}
 				datalength=payload_length;
 				// change _ptr to the new buffer
 				_ptr=dest;
@@ -521,7 +558,8 @@ int MySQL_Data_Stream::buffer2array() {
 				}
 			}
 			if (payload_length) {
-				l_free(destLen,dest);
+				//l_free(destLen,dest);
+				free(dest);
 			}
 			l_free(queueIN.pkt.size,queueIN.pkt.ptr);
 			pkts_recv++;
@@ -867,7 +905,15 @@ void MySQL_Data_Stream::setDSS_STATE_QUERY_SENT_NET() {
 void MySQL_Data_Stream::return_MySQL_Connection_To_Pool() {
 	MySQL_Connection *mc=myconn;
 	mc->last_time_used=sess->thread->curtime;
-	if ((mysql_thread___connection_max_age_ms) && (mc->last_time_used > mc->creation_time + mysql_thread___connection_max_age_ms * 1000)) {
+	// before detaching, check if last_HG_affected_rows matches . if yes, set it back to -1
+	if (mybe) {
+		if (mybe->hostgroup_id == sess->last_HG_affected_rows) {
+			sess->last_HG_affected_rows = -1;
+		}
+	}
+	unsigned long long intv = mysql_thread___connection_max_age_ms;
+	intv *= 1000;
+	if ((intv) && (mc->last_time_used > mc->creation_time + intv)) {
 		destroy_MySQL_Connection_From_Pool(true);
 	} else {
 		detach_connection();
